#include "vtkHorizontalAverage.h"

#include "vtkImageData.h"
#include "vtkObjectFactory.h"

vtkCxxRevisionMacro(vtkHorizontalAverage, "$Revision: 1.10 $");
vtkStandardNewMacro(vtkHorizontalAverage);

// The switch statement in Execute will call this method with
// the appropriate input type (IT). Note that this example assumes
// that the output data type is the same as the input data type.
// This is not always the case.
template <class IT>
void vtkHorizontalAverageExecute(vtkImageData* input,
                                        vtkImageData* output,
                                        IT* inPtr, IT* outPtr)
{
  int dims[3];
  input->GetDimensions(dims);
  if (input->GetScalarType() != output->GetScalarType())
    {
    vtkGenericWarningMacro(<< "Execute: input ScalarType, " << input->GetScalarType()
    << ", must match out ScalarType " << output->GetScalarType());
    return;
    }
  
  int size = dims[0]*dims[1]*dims[2];
 
  double horizontal[dims[0]];
  double orig;
  int index;
  double factor;
  //Calculate the Horiziontal Average
      for (int i=0; i<dims[0]; i++) { 
    	horizontal[i] = 0.0;
	for (int j=0; j<dims[1]; j++) {
	    for (int k=0; k<dims[2]; k++) {
		index = i*(dims[1]*dims[0]) + j*dims[0] + k;

		horizontal[i] += inPtr[index];
		
	    }
	}
	horizontal[i] = horizontal[i]/(dims[1]*dims[2]);
        
    }
    
    //subtract the Horizontal Average
    for (int i=0; i<dims[0]; i++) { 
    	for (int j=0; j<dims[1]; j++) {
	    for (int k=0; k<dims[2]; k++) {
		index = i*(dims[1]*dims[0]) + j*dims[0] + k;
		
		outPtr[index] = inPtr[index] - horizontal[i];
		printf("%f \n",outPtr[index]);
	    }
	}
    }
  
}

void vtkHorizontalAverage::SimpleExecute(vtkImageData* input,
                                                vtkImageData* output)
{

  void* inPtr = input->GetScalarPointer();
  void* outPtr = output->GetScalarPointer();

  switch(output->GetScalarType())
    {
    // This is simple a #define for a big case list. It handles
    // all data types vtk can handle.
    vtkTemplateMacro4(vtkHorizontalAverageExecute, input, output,(VTK_TT *)(inPtr), (VTK_TT *)(outPtr));
    default:
      vtkGenericWarningMacro("Execute: Unknown input ScalarType");
      return;
    }
}
